<?php

/*
 * This file is part of the Tajine software.
 * <https://gitlab.adullact.net/adullact/pki/tajine>
 *
 * Copyright (c) ADULLACT   <https://adullact.org>
 *               Association des Développeurs et Utilisateurs de Logiciels Libres
 *               pour les Administrations et les Collectivités Territoriales
 *
 * Tajine is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this software. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>.
 */

declare(strict_types=1);

namespace App\Domain\Certificate\Cfssl;

use Symfony\Component\HttpFoundation\Response;

class CfsslInfo
{
    public readonly string $cfsslApiMethodUrl ;
    public readonly string $cfsslApiJsonResponse ;
    public readonly string $cfsslApiJsonPayload ;
    public readonly int $curlCode ;
    public function __construct (
        public readonly string $cfsslServer = 'http://127.0.0.1:8888',
        public readonly string $cfsslApiRelativeUrl = 'api/v1/cfssl',
        public readonly string $cfsslApiEndpoint = 'info',
    ) {
        $this->cfsslApiMethodUrl = "$cfsslServer/$cfsslApiRelativeUrl/$cfsslApiEndpoint";
    }

    public function loadApiRequest (string $cfsslApiJsonPayload = "{}"):void {

        $cfsslUrl = $this->cfsslApiMethodUrl;
        $ch = curl_init($cfsslUrl);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($ch, CURLOPT_POSTFIELDS, "$cfsslApiJsonPayload");
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true); # Return response instead of printing.
        curl_setopt(
            $ch,
            CURLOPT_HTTPHEADER,
            array(
                'Content-Type:application/json',
                'Content-Length: ' . strlen("$cfsslApiJsonPayload")
            )
        );
        $result = curl_exec($ch);
        $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        switch ($http_code) {
            case 0:
                throw new \Exception("HTTP request failed - Bad CFSSL URL or CFSSL server is not up [ $cfsslUrl ]");
            case 404:
                throw new \Exception("Error HTTP 404 Not Found --> Bad CFSSL endpoind [ $cfsslUrl ]");
            default:
                break;
        }
        curl_close($ch);

        if ($result === false) {
            throw new \Exception("Error CFSSL-02 - HTTP request failed [ $cfsslUrl ]");
        }
        $json = json_decode($result);
        $this->curlCode = $http_code;
        $this->cfsslApiJsonPayload = $cfsslApiJsonPayload;
        $this->cfsslApiJsonResponse = $result;
        if (\is_null($json) | !isset($json->success)) {
            throw new \Exception("Error CFSSL-03 - Loading Json failed: [ HTTP $http_code ]");
        }


        if ($json->success !== true && $http_code === 405) {
            throw new \Exception("Error HTTP 405 Method Not Allowed --> Method is not allowed");
        }
        else if ($json->success !== true && $http_code === 400) {
            // @@@TODO add CFSSL error message
            throw new \Exception("Error HTTP 400 Bad Request --> Json payload is not valid");
        }
        else if ($json->success !== true) {
            throw new \Exception("Error CFSSL-03 - Unexpected HTTP code: [ $http_code ]");
        }
        else { // $json->success === true + HTTP 200

        }
    }

}
